//--------------------------------------------------------------------
// $Id: Array.hpp 4721 2007-09-15 04:03:10Z cjm $
//--------------------------------------------------------------------
//
//   GUEmap
//   Copyright 1997-2007 by Christopher J. Madsen
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License along
//   with this program; if not, write to the Free Software Foundation, Inc.,
//   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//   Template class for dynamic array (based on STL vector)
//
//--------------------------------------------------------------------

#ifndef __ARRAY_HPP

#define __ARRAY_HPP

template <class Type, class TVec>
class Array
{
 private:
  TVec  v;
 public:
  Array() {};
  Array(TVec& v2) { v.swap(v2); };
  ~Array() { Array::removeAll(); };
  Type&        operator[](VecSize pos)
    { ASSERT(pos >= 0 && pos < v.size()); return *v[pos]; };
  const Type&  operator[](VecSize pos) const
    { ASSERT(pos >= 0 && pos < v.size()); return *v[pos]; };
  bool         empty()                 const { return v.empty(); };
  Type*        extract(VecSize pos)
    {
      ASSERT(pos >= 0 && pos < v.size());
      Type* r = v[pos]; v.erase(v.begin() + pos); return r;
    };
  void         extractAll(TVec& v2) { v2.clear(); v.swap(v2); };
  Type*        getPtr(VecSize pos)
    { ASSERT(pos >= 0 && pos < v.size()); return v[pos]; };
  const Type*  getPtr(VecSize pos) const
    { ASSERT(pos >= 0 && pos < v.size()); return v[pos]; };
  TVec&        getVector()       { return v; };
  const TVec&  getVector() const { return v; };
  void     append(Type* obj) { v.insert(v.end(), obj); };
  void     insert(VecSize pos, Type* obj)
    { ASSERT(pos >= 0 && pos <= v.size()); v.insert(v.begin() + pos, obj); };
  void     remove(VecSize pos)
    {
      ASSERT(pos >= 0 && pos < v.size());
      delete v[pos]; v.erase(v.begin() + pos);
    };
  TVec::iterator  remove(TVec::iterator itr)
    { delete *itr; return v.erase(itr); };
  void     removeAll()
  { TVec::iterator p = v.begin(); while (p != v.end()) { delete *(p++); }
    v.clear(); };
  void     reserve(VecSize n) { v.reserve(n); };
  void     resize(VecSize n)
    { ASSERT(v.size() <= n); v.insert(v.end(), n - v.size(), NULL); };
  VecSize  size() const { return v.size(); };
  Array<Type,TVec>& operator=(const Array<Type,TVec>& source)
    {
      if (this != &source) {
        removeAll();
        v.reserve(source.v.size());
        for (TVec::const_iterator e=source.v.begin(); e != source.v.end(); e++)
          append(new Type(**e));
      }
      return *this;
    };
  bool  operator==(const Array<Type,TVec>& x) const
    {
      if (v.size() != x.v.size()) return false;
      for (TVec::const_iterator a=v.begin(), b=x.v.begin();
           a != v.end();
           ++a, ++b)
        if (**a != **b) return false;
      return true;
    };
}; // end Array

#endif // __ARRAY_HPP
