//--------------------------------------------------------------------
// $Id: CommentDlg.cpp 4718 2007-09-15 02:34:13Z cjm $
//--------------------------------------------------------------------
//
//   GUEmap
//   Copyright 1997-2007 by Christopher J. Madsen
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License along
//   with this program; if not, write to the Free Software Foundation, Inc.,
//   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//   Free floating room comments dialog
//
//--------------------------------------------------------------------

#include "StdAfx.hpp"
#include "GUEmap.hpp"
#include "CommentDlg.hpp"
#include "MapDoc.hpp"
#include "MapView.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// Class CCommentDlg:
//
// Member Variables:
//   active:       TRUE means the dialog is the active window
//   comment:      Contains the current map or room comment
//   roomComment:  FALSE means map comment, TRUE means room comment
//   fromView:     Pointer to the view that posted this comment
//--------------------------------------------------------------------
// CCommentDlg construction/destruction:

CCommentDlg::CCommentDlg(CWnd* pParent /*=NULL*/)
: CDialog(CCommentDlg::IDD, pParent),
  fromView(NULL),
  pos(0,0,0,0),
  active(false),
  roomComment(false)
{
	//{{AFX_DATA_INIT(CCommentDlg)
	comment = _T("");
	//}}AFX_DATA_INIT
} // end CCommentDlg::CCommentDlg

void CCommentDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCommentDlg)
	DDX_Text(pDX, IDC_CD_COMMENT, comment);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CCommentDlg, CDialog)
	//{{AFX_MSG_MAP(CCommentDlg)
	ON_WM_ACTIVATE()
	ON_WM_CLOSE()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	ON_COMMAND(IDCANCEL, OnCancel)
END_MESSAGE_MAP();

/////////////////////////////////////////////////////////////////////////////
// CCommentDlg message handlers
//--------------------------------------------------------------------
// Update the room comment when we lose focus:

void CCommentDlg::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized)
{
  if (nState == WA_INACTIVE) {
    active = false;
    if (fromView) {
      UpdateData(TRUE);
      if (roomComment)
        fromView->setRoomNote(comment);
      else
        fromView->GetDocument()->setNote(comment);
    }
  } else
    active = true;

  CDialog::OnActivate(nState, pWndOther, bMinimized);
} // end CCommentDlg::OnActivate

//--------------------------------------------------------------------
// Cancel any changes made to the text but not yet saved:

void CCommentDlg::OnCancel()
{
  UpdateData(FALSE);
} // end CCommentDlg::OnCancel

//--------------------------------------------------------------------
// Close the dialog:

void CCommentDlg::OnClose()
{
  // Tell the application to close the comments dialog:
  static_cast<CMapApp*>(AfxGetApp())->OnViewComments();
} // end CCommentDlg::OnClose

//--------------------------------------------------------------------
// Resize the edit box when the dialog is resized:

void CCommentDlg::OnSize(UINT nType, int cx, int cy)
{
  CDialog::OnSize(nType, cx, cy);

  HWND  ec;
  GetDlgItem(IDC_CD_COMMENT, &ec);
  if (ec != NULL)
    ::SetWindowPos(ec, HWND_TOP, 0,0, cx,cy, SWP_NOMOVE|SWP_NOZORDER);
} // end CCommentDlg::OnSize

//--------------------------------------------------------------------
// Make Tab switch focus back to the map:

BOOL CCommentDlg::PreTranslateMessage(MSG* pMsg)
{
  if (pMsg->message == WM_KEYDOWN) {
    if (pMsg->wParam == VK_TAB  &&
        GetAsyncKeyState(VK_CONTROL) >= 0) { // Control is not pressed
      AfxGetMainWnd()->SetFocus();
      return TRUE;
    } else if (pMsg->wParam == 'F' &&
               GetAsyncKeyState(VK_CONTROL) < 0) { // Control is pressed
      gueApp()->OnEditFind();
      return TRUE;
    }
  } // end if WM_KEYDOWN
  else if (pMsg->message == WM_SYSKEYDOWN &&
           (pMsg->wParam == 'F' || pMsg->wParam == 'Q')) {
    CEdit*  edit = static_cast<CEdit*>(GetDlgItem(IDC_CD_COMMENT));
    CString  text;
    edit->GetWindowText(text);
    String s;
    Strcpy(s, text);
    fillParagraphs(s);
    if (s != LPCTSTR(text)) {
      edit->SetSel(0, -1, TRUE);
      edit->ReplaceSel(s.c_str(), TRUE); // Can undo
    }
    return TRUE;
  } // end if WM_SYSKEYDOWN

  if (CDialog::PreTranslateMessage(pMsg))
    return TRUE;
  else
    return FALSE;
} // end CCommentDlg::PreTranslateMessage

//////////////////////////////////////////////////////////////////////
// Local Variables:
// compile-command: "nmake -f GUEmap.mak"
// End:
