//--------------------------------------------------------------------
// $Id: File.hpp 4718 2007-09-15 02:34:13Z cjm $
//--------------------------------------------------------------------
//
//   GUEmap
//   Copyright 1997-2007 by Christopher J. Madsen
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License along
//   with this program; if not, write to the Free Software Foundation, Inc.,
//   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//   File constants and interface of the FileReader class
//
//--------------------------------------------------------------------

#ifndef __FILE_H

#define __FILE_H

//====================================================================
// Codes for GUEmap files:
//--------------------------------------------------------------------
// Normal codes:

const BYTE
  fcSizeMask    = 0007,
  fcStartRecord = 0011,
  fcEndRecord   = 0000,

//--------------------------------------------------------------------
// Record specific codes:
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// Map Information:

  fcMapNumRooms = 0022,
  fcMapNumEdges = 0032,
  fcMapName     = 0040,
  fcMapNote     = 0050,
  fcMapSize     = 0064,
  fcMapNumPages = 0071,

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// Page:

  fcPagePosition = 0024,

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// Room:

  fcRoomName	 = 0020,
  fcRoomPosition = 0034,
  fcRoomNote     = 0040,
  fcRoomFlags    = 0051,

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// Edge:

  fcEdgeEnd1	= 0022,
  fcEdgeEnd1L	= 0023,
  fcEdgeEnd2	= 0032,
  fcEdgeEnd2L	= 0033,
  fcEdgeType	= 0042,

//--------------------------------------------------------------------
// Record codes:

  recMap	= 1,
  recRoom	= 2,
  recEdge	= 3,
  recPage	= 4;

//====================================================================
// Miscellaneous functions:

CArchive& operator<<(CArchive& ar, const String& s);
CArchive& operator>>(CArchive& ar, String& s);

//====================================================================
// Class declarations:

class FileReader
{
private:
  CArchive&  ar;
  int        rcCache;
public:
  FileReader(CArchive& archive);
  BYTE  byte()    { BYTE b; ar >> b; return b; };
  bool  boolean() { return bool(byte());       };
  void  findRecord(BYTE type);
  BYTE  peekNextRecord();
  void  skipField(BYTE fieldCode);
  WORD  word()    { WORD w; ar >> w; return w; };
}; // end FileReader

#endif // __FILE_H

// Local Variables:
// mode: c++
// compile-command:"nmake -f GUEmap.mak"
// End:
