//--------------------------------------------------------------------
// $Id: GUEmap.cpp 4718 2007-09-15 02:34:13Z cjm $
//--------------------------------------------------------------------
//
//   GUEmap
//   Copyright 1996-2007 by Christopher J. Madsen
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License along
//   with this program; if not, write to the Free Software Foundation, Inc.,
//   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//   GUEmap.cpp : Defines the class behaviors for the application
//
//--------------------------------------------------------------------

#include "StdAfx.hpp"
#include "GUEmap.hpp"

#include <winspool.h>           // For printer settings

#include "MainFrm.hpp"
#include "ChildFrm.hpp"
#include "CommentDlg.hpp"
#include "FindDlg.hpp"
#include "MapDoc.hpp"
#include "MapView.hpp"
#include "NavOpt.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

bool     GUEmapEatClicks = true;
HCURSOR  handCursor = NULL;

const char *const iniCDlg        = "CommentDialog";
const char *const iniMainWin     = "MainWindow";
const char *const iniWinLeft     = "Left";
const char *const iniWinRight    = "Right";
const char *const iniWinTop      = "Top";
const char *const iniWinBottom   = "Bottom";

const char *const iniNavigation  = "Navigation";
const char *const iniNavCopy     = "AutoCopy";
const char *const iniNavCRLF     = "AddCRLF";
const char *const iniNavEdit     = "AutoEdit";
const char *const iniNavStub     = "PreferUnexplored";

/////////////////////////////////////////////////////////////////////////////
// Miscellaneous functions:
//--------------------------------------------------------------------
// Copy a string, replacing CRLF with space:
//
// Assumes that any CR is followed by a LF.
//
// Input:
//   source:  The string to copy
//
// Output:
//   dest:    The copied string, with any CRLF replaced by a space

void copyToOneLine(String& dest, const String& source)
{
  dest = source;
  StrIdx  p = 0;
  while ((p = dest.find(_T('\r'),p)) != String::npos) {
    dest.erase(p,1);
    dest[p] = _T(' ');
  }
} // end copyToOneLine

//--------------------------------------------------------------------
// Fill paragraphs in a string:
//
// Removes all leading and trailing space, then removes CRLF pairs
// unless they are on a blank line or followed by a space.  Assumes
// that any CR is followed by a LF.
//
// Input:
//   s:  The string to fill
//
// Output:
//   s:  The filled string

void fillParagraphs(String& s)
{
  trimRight(s);
  trimLeft(s);

  StrIdx  p = 0;
  // Convert tabs to spaces (just in case):
  while ((p = s.find(_T('\t'),p)) != String::npos)
    s[p] = _T(' ');

  p = 0;
  while ((p = s.find(_T('\r'),p)) != String::npos) {
    if ((s[p-1] != _T('\n')) &&                       // Not a blank line &
        s.find_first_not_of(_T("\n\r "), p) == p+2) { // next char not a space
      StrIdx  len = 1;
      while (s[p-1] == _T(' ')) {
        --p;
        ++len;
      } // end while spaces precede this CR
      s.erase(p,len);           // Erase the CR and preceding spaces
      s[p] = _T(' ');           // Change the LF to a space
    } else
      ++p; // Don't remove this CRLF
  } // end while more CRs
} // end fillParagraphs

//--------------------------------------------------------------------
// Set a window's position:
//
// Input:
//   pos:  The new position of the window (in screen coordinates)
//   w:    The window to move
//
// Note:
//   If pos.right is 0, or any part of the window is outside the
//   current desktop, then we don't move the window.

void setWindowPos(const CRect& pos, CWnd* w)
{
  if (pos.right && (pos.left < pos.right) && (pos.top < pos.bottom)) {
    CRect desktop;
    ::GetWindowRect(::GetDesktopWindow(),&desktop);
    if ((pos.left   >= desktop.left)  &&
        (pos.right  <= desktop.right) &&
        (pos.top    >= desktop.top)   &&
        (pos.bottom <= desktop.bottom))
      w->MoveWindow(&pos);
  }
} // end setWindowPos

//--------------------------------------------------------------------
// Trim whitespace from left of string:

void trimLeft(String& s)
{
  StrIdx  p = s.find_first_not_of(_T("\n\r\t "));
  if (p > 0) s.erase(0, p);
} // end trimLeft

//--------------------------------------------------------------------
// Trim whitespace from right of string:

void trimRight(String& s)
{
  StrIdx  p = s.find_last_not_of(_T("\n\r\t "));
  if (p < s.length()) s.erase(p+1);
} // end trimRight

/////////////////////////////////////////////////////////////////////////////
// CMapApp

BEGIN_MESSAGE_MAP(CMapApp, CWinApp)
	//{{AFX_MSG_MAP(CMapApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_EDIT_FIND, OnEditFind)
	ON_COMMAND(ID_VIEW_COMMENTS, OnViewComments)
	ON_COMMAND(ID_VIEW_SETTINGS, OnViewSettings)
	ON_UPDATE_COMMAND_UI(ID_VIEW_COMMENTS, OnUpdateViewComments)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
	// Standard help commands (see also MainFrame.cpp)
	ON_COMMAND(ID_HELP_INDEX, CWinApp::OnHelpIndex)
END_MESSAGE_MAP();

/////////////////////////////////////////////////////////////////////////////
// CMapApp construction

CMapApp::CMapApp()
: clipboard(NULL),
  commentDlg(NULL),
  findDlg(NULL),
  initialized(false)
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

CMapApp::~CMapApp()
{
  delete clipboard;
  delete commentDlg;
  delete findDlg;
} // end CMapApp::~CMapApp

/////////////////////////////////////////////////////////////////////////////
// The one and only CMapApp object

CMapApp theApp;
String  mapWinClass;

/////////////////////////////////////////////////////////////////////////////
// CMapApp initialization

BOOL CMapApp::InitInstance()
{
  // Standard initialization
  // If you are not using these features and wish to reduce the size
  //  of your final executable, you should remove from the following
  //  the specific initialization routines you do not need.

#ifdef _AFXDLL
  Enable3dControls();           // Call this when using MFC in a shared DLL
#else
  Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

  mapWinClass = AfxRegisterWndClass(
    CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW,
    LoadStandardCursor(IDC_ARROW), // Standard cursor
    (HBRUSH)::GetStockObject(WHITE_BRUSH)
  );

  handCursor = LoadCursor(IDC_HAND);

  LoadStdProfileSettings(9); // Load standard INI file options (including MRU)

  // Register the application's document templates.  Document templates
  //  serve as the connection between documents, frame windows and views.

  CMultiDocTemplate* pDocTemplate;
  pDocTemplate = new CMultiDocTemplate(
    IDR_GUEMAPTYPE,
    RUNTIME_CLASS(CMapDoc),
    RUNTIME_CLASS(CChildFrame), // custom MDI child frame
    RUNTIME_CLASS(CMapView));
  AddDocTemplate(pDocTemplate);

  // create room comments dialog
  if (!(commentDlg = new CCommentDlg()))
    return FALSE;

  loadWindowPos(commentDlg->pos, iniCDlg);

  // create main MDI Frame window
  CMainFrame* pMainFrame = new CMainFrame;
  if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
    return FALSE;
  m_pMainWnd = pMainFrame;

  // Enable file manager drag/drop and DDE Execute open
  EnableShellOpen();
  RegisterShellFileTypes(TRUE);

  // Parse command line for standard shell commands, DDE, file open
  CCommandLineInfo cmdInfo;
  ParseCommandLine(cmdInfo);

  // Dispatch commands specified on the command line
  if (!ProcessShellCommand(cmdInfo))
    return FALSE;

  // The main window has been initialized, so show and update it.
  CRect  pos;
  loadWindowPos(pos, iniMainWin);
  setWindowPos(pos, pMainFrame);
  pMainFrame->ShowWindow(m_nCmdShow);
  pMainFrame->UpdateWindow();
  pMainFrame->DragAcceptFiles(TRUE);

  setLandscape();               // Switch to landscape mode

  // Load navigation options:
  autoEdit = GetProfileInt(iniNavigation,iniNavEdit,true);
  naviCopy = GetProfileInt(iniNavigation,iniNavCopy,true);
  naviCRLF = GetProfileInt(iniNavigation,iniNavCRLF,false);
  preferUnexplored = GetProfileInt(iniNavigation,iniNavStub,true);

  initialized = true;
  return TRUE;
} // end CMapApp::InitInstance

//--------------------------------------------------------------------
// Load a window position from the INI file:
//
// Input:
//   section:  The title of the section to load it from
//
// Output:
//   pos:  The window position

void CMapApp::loadWindowPos(CRect& pos, LPCTSTR section)
{
  pos.left   = GetProfileInt(section, iniWinLeft,   0);
  pos.right  = GetProfileInt(section, iniWinRight,  0);
  pos.top    = GetProfileInt(section, iniWinTop,    0);
  pos.bottom = GetProfileInt(section, iniWinBottom, 0);
} // end CMapApp::loadWindowPos

//--------------------------------------------------------------------
// Set landscape mode as default:
//
// From KB: Q126897 (1-21-96)

void CMapApp::setLandscape()
{
  // Get default printer settings.
  PRINTDLG   pd;

  pd.lStructSize = (DWORD) sizeof(PRINTDLG);
  if (GetPrinterDeviceDefaults(&pd)) {
    // Lock memory handle.
    DEVMODE FAR* pDevMode =
      (DEVMODE FAR*)::GlobalLock(m_hDevMode);
    LPDEVNAMES lpDevNames;
    LPTSTR lpszDriverName, lpszDeviceName, lpszPortName;
    HANDLE hPrinter;

    if (pDevMode) {
      // Change printer settings in here.
      pDevMode->dmOrientation = DMORIENT_LANDSCAPE;
      // Unlock memory handle.
      lpDevNames = (LPDEVNAMES)GlobalLock(pd.hDevNames);
      lpszDriverName = (LPTSTR )lpDevNames + lpDevNames->wDriverOffset;
      lpszDeviceName = (LPTSTR )lpDevNames + lpDevNames->wDeviceOffset;
      lpszPortName   = (LPTSTR )lpDevNames + lpDevNames->wOutputOffset;

      ::OpenPrinter(lpszDeviceName, &hPrinter, NULL);
      ::DocumentProperties(NULL,hPrinter,lpszDeviceName,pDevMode,
                           pDevMode, DM_IN_BUFFER|DM_OUT_BUFFER);

      // Sync the pDevMode.
      // See SDK help for DocumentProperties for more info.
      ::ClosePrinter(hPrinter);
      ::GlobalUnlock(m_hDevNames);
      ::GlobalUnlock(m_hDevMode);
    }
  }
} // end CMapApp::setLandscape

/////////////////////////////////////////////////////////////////////////////
// CMapApp shutdown:

int CMapApp::ExitInstance()
{
  if (initialized) { // We finished initialization
    WriteProfileInt(iniNavigation, iniNavEdit, autoEdit);
    WriteProfileInt(iniNavigation, iniNavCopy, naviCopy);
    WriteProfileInt(iniNavigation, iniNavCRLF, naviCRLF);
    WriteProfileInt(iniNavigation, iniNavStub, preferUnexplored);

    if (commentDlg) {
      if (commentDlg->GetSafeHwnd())
        OnViewComments();       // Destroy comment window
      saveWindowPos(commentDlg->pos, iniCDlg);
    }

    if (findDlg && findDlg->GetSafeHwnd())
      findDlg->DestroyWindow();
  }

  return CWinApp::ExitInstance();
} // end CMapApp::ExitInstance

//--------------------------------------------------------------------
// Save a window position to the INI file:
//
// Input:
//   pos:      The window position
//   section:  The title of the section to save it in

void CMapApp::saveWindowPos(LPCRECT pos, LPCTSTR section)
{
  WriteProfileInt(section, iniWinLeft,   pos->left);
  WriteProfileInt(section, iniWinRight,  pos->right);
  WriteProfileInt(section, iniWinTop,    pos->top);
  WriteProfileInt(section, iniWinBottom, pos->bottom);
} // end CMapApp::saveWindowPos

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
 public:
  CAboutDlg();

  // Dialog Data
  //{{AFX_DATA(CAboutDlg)
  enum { IDD = IDD_ABOUTBOX };
  CString	username;
  //}}AFX_DATA

  // ClassWizard generated virtual function overrides
  //{{AFX_VIRTUAL(CAboutDlg)
 protected:
  virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
  //}}AFX_VIRTUAL

  // Implementation
 protected:
  //{{AFX_MSG(CAboutDlg)
  //}}AFX_MSG
  DECLARE_MESSAGE_MAP();
}; // end CAboutDlg

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	username = _T("");
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP();

// App command to run the dialog
void CMapApp::OnAppAbout()
{
  CAboutDlg aboutDlg;
  aboutDlg.DoModal();
} // end CMapApp::OnAppAbout

/////////////////////////////////////////////////////////////////////////////
// CMapApp commands
//--------------------------------------------------------------------
// Find text in comments:

void CMapApp::OnEditFind()
{
  if (!findDlg) {
    VERIFY(findDlg = new CFindDlg(commentDlg));
    if (!findDlg) return;
  }

  if (findDlg->GetSafeHwnd() != NULL) {
    findDlg->SetActiveWindow();
    findDlg->GotoDlgCtrl(findDlg->GetDlgItem(IDC_FIND_WHAT));
  } else {
    findDlg->Create();
    setWindowPos(findDlg->pos, findDlg);
  }
} // end CMapApp::OnEditFind

//====================================================================
// The Room Comments dialog:
//--------------------------------------------------------------------
// Display or hide the free-floating Room Comments dialog:

void CMapApp::OnViewComments()
{
  if (commentDlg->GetSafeHwnd() != NULL) {
    commentDlg->GetWindowRect(&commentDlg->pos);
    commentDlg->DestroyWindow();
  } else {
    commentDlg->Create();
    setWindowPos(commentDlg->pos, commentDlg);
    commentDlg->SetWindowText(commentDlg->title.empty()
                              ? _T("Room Comments")
                              : commentDlg->title.c_str());
    commentDlg->SendDlgItemMessage(IDC_CD_COMMENT,WM_ENABLE,
                                   commentDlg->fromView != NULL);
    commentDlg->SendDlgItemMessage(IDC_CD_COMMENT,EM_SETREADONLY,
                                   commentDlg->fromView == NULL);
    commentDlg->SendDlgItemMessage(IDC_CD_COMMENT,EM_SETSEL, -1, -2);
  }
} // end CMapApp::OnViewComments

void CMapApp::OnUpdateViewComments(CCmdUI* pCmdUI)
{
  pCmdUI->SetCheck(commentDlg->GetSafeHwnd() != NULL);
} // end CMapApp::OnUpdateViewComments

//--------------------------------------------------------------------
// Move the Room Comments dialog so it doesn't obscure the map:
//
// Input:
//   mapPos:
//     The screen coordinates that should not be obscured
//     Must be normalized

void CMapApp::adjustCommentPos(LPCRECT mapPos)
{
  if (commentDlg->GetSafeHwnd()) {
    CRect  dlgPos, r;
    commentDlg->GetWindowRect(&dlgPos);
    if (r.IntersectRect(dlgPos, mapPos)) {
      CRect  mainPos;
      m_pMainWnd->GetClientRect(&mainPos);
      m_pMainWnd->ClientToScreen(&mainPos);
      BOOL max;
      const CWnd* curWin =
        static_cast<CMainFrame*>(m_pMainWnd)->MDIGetActive(&max);
      if (max) {
        curWin = curWin->GetTopWindow(); // Get the active view
        curWin->GetClientRect(&r);       // and make sure we don't
        curWin->ClientToScreen(&r);      // cover up the scroll bars
        mainPos.bottom = r.bottom;       // on the right and bottom
        mainPos.right  = r.right;
      } // end if active window is maximized
      dlgPos.OffsetRect(mainPos.right - dlgPos.right, // Top right
                        mainPos.top - dlgPos.top);
      if (!r.IntersectRect(dlgPos, mapPos)) goto move;
      dlgPos.OffsetRect(mainPos.right - dlgPos.right, // Bottom right
                        mainPos.bottom - dlgPos.bottom);
      if (!r.IntersectRect(dlgPos, mapPos)) goto move;
      dlgPos.OffsetRect(mainPos.left - dlgPos.left,   // Bottom left
                        mainPos.bottom - dlgPos.bottom);
      if (!r.IntersectRect(dlgPos, mapPos)) goto move;
      dlgPos.OffsetRect(mainPos.left - dlgPos.left,   // Top left
                        mainPos.top - dlgPos.top);
      if (r.IntersectRect(dlgPos, mapPos)) return;  // Give up
     move:
      commentDlg->MoveWindow(dlgPos);
    } // end if area is obscured
  } // end if commentDlg is visible
} // end CMapApp::adjustCommentPos

//--------------------------------------------------------------------
// Disable comment dialog when closing its view:
//
// Input:
//   view:  The view that is closing

void CMapApp::closingView(const CMapView* view)
{
  if (commentDlg->fromView == view)
    setComment(NULL, NULL);
} // end CMapApp::closingView

//--------------------------------------------------------------------
// Switch to the Room Comments dialog:

void CMapApp::editComment()
{
  if (commentDlg->GetSafeHwnd() == NULL)
    OnViewComments();
  else {
    commentDlg->SetFocus();
    commentDlg->SendDlgItemMessage(IDC_CD_COMMENT, EM_SETSEL, -1, -2);
  }
} // end CMapApp::editComment

//--------------------------------------------------------------------
// Update the Room Comments dialog:
//
// Input:
//   fromView:
//     The map view which is posting this comment
//     NULL means the view is closing, so disable the dialog
//   room:
//     The room whose comment should be displayed in the dialog
//     NULL means no room is selected, so display map comment
//   takeFocus: (default true)
//     TRUE means to override a comment from a different view

void CMapApp::setComment(CMapView* fromView, const MapRoom* room,
                         bool takeFocus) const
{
  if (!commentDlg->active &&
      (takeFocus || (commentDlg->fromView == fromView))) {
    commentDlg->fromView = fromView;
    if (room && !(room->flags & rfCorner)) {
      Strcpy(commentDlg->comment, room->note);
      copyToOneLine(commentDlg->title, room->name);
      commentDlg->roomComment = true;
    } else {
      if (fromView) {
        const CMapDoc*  doc = fromView->GetDocument();
        ASSERT_VALID(doc);

        Strcpy(commentDlg->comment, doc->getNote());
        commentDlg->title = doc->getName();
        if (!commentDlg->title.empty())
          commentDlg->title += _T(" -- ");
      } else {
        commentDlg->comment.Empty();
        commentDlg->title.erase();
      } // end else no active view
      commentDlg->title += _T("Map Comments");
      commentDlg->roomComment = false;
    } // end else no current room
    if (commentDlg->GetSafeHwnd()) {
      commentDlg->UpdateData(FALSE);
      commentDlg->SetWindowText(commentDlg->title.empty()
                                ? _T("Room Comments")
                                : commentDlg->title.c_str());
      commentDlg->SendDlgItemMessage(IDC_CD_COMMENT,WM_ENABLE,
                                     fromView != NULL);
      commentDlg->SendDlgItemMessage(IDC_CD_COMMENT,EM_SETREADONLY,
                                     fromView == NULL);
    } // end if comment dialog is open
  } // end if we should update the comment
} // end CMapApp::setComment

//====================================================================
// CMapApp Options dialog:
//--------------------------------------------------------------------
// Bring up the Options dialog:

void CMapApp::OnViewSettings()
{
  editOptions();
} // end CMapApp::OnViewSettings

//--------------------------------------------------------------------
// Bring up the Options dialog:

void CMapApp::editOptions()
{
  CPropertySheet dlg(IDR_OPTIONS);

  COptNavPage  nav;
  nav.copy = naviCopy;
  nav.CRLF = naviCRLF;
  nav.editProps = autoEdit;
  nav.stubs = (preferUnexplored ? 0 : 1);
  dlg.AddPage(&nav);

  if (dlg.DoModal() == IDOK) {
    autoEdit = nav.editProps;
    naviCopy = nav.copy;
    naviCRLF = nav.CRLF;
    preferUnexplored = (nav.stubs == 0);
  } // end if OK pressed
} // end CMapApp::editOptions

/////////////////////////////////////////////////////////////////////////////
// Local Variables:
// compile-command: "nmake -f GUEmap.mak"
// End:
