//--------------------------------------------------------------------
// $Id: GUEmap.hpp 4718 2007-09-15 02:34:13Z cjm $
//--------------------------------------------------------------------
//
//   GUEmap
//   Copyright 1996-2007 by Christopher J. Madsen
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License along
//   with this program; if not, write to the Free Software Foundation, Inc.,
//   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//   GUEmap.h : main header file for the GUEMAP application
//
//--------------------------------------------------------------------

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"       // main symbols

/////////////////////////////////////////////////////////////////////////////
// Miscellaneous functions:

void copyToOneLine(String& dest, const String& source);
void fillParagraphs(String& s);
void trimLeft(String& s);
void trimRight(String& s);

/////////////////////////////////////////////////////////////////////////////
// CMapApp:
// See GUEmap.cpp for the implementation of this class
//

class CAboutDlg;
class CCommentDlg;
class CFindDlg;
class CMapView;
class MapRoom;
class RoomScrap;

extern bool  GUEmapEatClicks;
extern const char *const iniMainWin;
extern HCURSOR handCursor;
extern String  mapWinClass;

class CMapApp : public CWinApp
{
 protected:
  bool  autoEdit;
  bool  preferUnexplored;
  bool  initialized;
  bool  naviCopy;
  bool  naviCRLF;
  CString username;
  CString password;
  RoomScrap*  clipboard;
  CCommentDlg*  commentDlg;
  CFindDlg*     findDlg;

 public:
  CMapApp();
  ~CMapApp();

  void adjustCommentPos(LPCRECT r);
  void closingView(const CMapView* view);
  void editComment();
  void editOptions();
  const RoomScrap*  getClipboard() const { return clipboard; };
  void loadWindowPos(CRect& pos, LPCTSTR section);
  bool navigationCopy() const { return naviCopy; };
  bool navigationCRLF() const { return naviCRLF; };
  bool editAfterAdd()   const { return autoEdit; };
  bool stub1Unexplored()const { return preferUnexplored; };
  void saveWindowPos(LPCRECT pos, LPCTSTR section);
  void setClipboard(RoomScrap* newScrap);
  void setComment(CMapView* fromView, const MapRoom* room,
                  bool takeFocus=true) const;
  void setLandscape();
  void setupAbout(CAboutDlg& dlg) const;

  // Overrides
  // ClassWizard generated virtual function overrides
  //{{AFX_VIRTUAL(CMapApp)
	public:
  virtual BOOL InitInstance();
  virtual int ExitInstance();
	//}}AFX_VIRTUAL

  // Implementation

	//{{AFX_MSG(CMapApp)
	afx_msg void OnAppAbout();
	afx_msg void OnEditFind();
	afx_msg void OnUpdateViewComments(CCmdUI* pCmdUI);
	afx_msg void OnViewComments();
	afx_msg void OnViewSettings();
	//}}AFX_MSG
  DECLARE_MESSAGE_MAP();
}; // end CMapApp

inline CMapApp* gueApp() { return static_cast<CMapApp*>(AfxGetApp()); }

/////////////////////////////////////////////////////////////////////////////
// Local Variables:
// mode: c++
// compile-command: "nmake -f GUEmap.mak"
// End:
