//--------------------------------------------------------------------
// $Id: MapView.hpp 4718 2007-09-15 02:34:13Z cjm $
//--------------------------------------------------------------------
//
//   GUEmap
//   Copyright 1996-2007 by Christopher J. Madsen
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License along
//   with this program; if not, write to the Free Software Foundation, Inc.,
//   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//   MapView.h : interface of the CMapView class
//
//--------------------------------------------------------------------

#include "ScrollZoom.hpp"

enum GmOp
{
  gmoNone, gmoAddEdge, gmoChangeEdge, gmoSelectBox,
  gmoDeleting, gmoAddCorner,
  gmoControlDown, gmoShiftDown,
  gmoShiftDragUp, gmoShiftDragDown, gmoShiftDragLeft, gmoShiftDragRight,
  gmoControlDrag,
  gmoDrag = gmoShiftDragUp      // Drag operations must come last
}; // end GmOp

enum EditPropUndo
{
  epuRoomInfo, epuAddRoom, epuAddRoomEdge, epuMapInfo
}; // end EditPropUndo

class CMapView : public CScrollZoomView
{
 protected: // create from serialization only
  CMapView();
  DECLARE_DYNCREATE(CMapView);

  // Attributes
  GmOp     opInProgress;
  bool     bTmp;
  bool     b2Tmp;
  MapEdge  edgeTmp;
  MapEdge  e2Tmp;
  int      iTmp;
  CPoint   p1Tmp;
  CPoint   p2Tmp;
  CRect    rTmp;

  CPoint   scrollDragStart;
  UINT     scrollDragTimer;
  bool     scrollDrag;

  CFont    font;
  bool     showCorners;
  bool     showGrid;
  bool     showPages;

  int      numSelected;
  int      numSelectedPages;
  int      selectedOne;
  ByteVec  selected;
  ByteVec  selectedPage;

  short    printingPage;
  short    zoom;

 public:
  const CMapDoc* GetDocument() const;
  CMapDoc* GetDocument();
  short    getZoom() const { return zoom; };

  // Operations
 public:
  void  clearSelection(bool update=true);
  void  deleteSelection(bool removeCorner=true);
  void  deselectPage(int n, bool update=true);
  void  deselectPages(bool update=true);
  void  deselectRoom(RoomNum n, bool update=true);
  void  selectDone();
  void  selectPage(int n, bool update=true);
  void  selectRoom(RoomNum n, bool update=true);
  void  selectOnlyRoom(RoomNum n)
  { if (selectedOne != n) { clearSelection(); selectRoom(n); selectDone(); } };

  void  makeRoomVisible(RoomNum n);
  void  navigate(RoomCorner corner, bool toggleStubs=false);
  void  setRoomNote(LPCTSTR comment);

  // Overrides
  // ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMapView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual void OnInitialUpdate();
	virtual void OnPrepareDC(CDC* pDC, CPrintInfo* pInfo = NULL);
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	//}}AFX_VIRTUAL

  // Implementation
 public:
  virtual ~CMapView();
#ifdef _DEBUG
  virtual void AssertValid() const;
  virtual void Dump(CDumpContext& dc) const;
#endif

 protected:

  void  addRoom(CPoint& point);
  void  adjustOffset(CPoint& p) const;
  void  computeSelectedRect(CRect& r) const;

  void  drawBackground(CDC* dc);
  void  drawSelected(CDC* dc);

  void  editProperties(EditPropUndo undoType, bool wasModified=false,
                       EdgeVec* edges=NULL);

  bool  pasteClipboard(const String& text);

  void  getNavText(char& initial, RoomCorner& dir1,
                   char& separator, RoomCorner& dir2,
                   bool mustBeMove=false);
  void  setEdgeType(MapEdge& e);
  void  setNavText(char c);

  void  setScrollBars();
  void  wrapRoomName(CString& name);
  void  zoomTo(short newZoom);

  // Generated message map functions
 protected:
	//{{AFX_MSG(CMapView)
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnEditAddCorner();
	afx_msg void OnEditAddPage();
	afx_msg void OnEditAddRoom();
	afx_msg void OnEditClear();
	afx_msg void OnEditCopy();
	afx_msg void OnEditCut();
	afx_msg void OnEditPaginate();
	afx_msg void OnEditPaste();
	afx_msg void OnEditSelectAll();
	afx_msg void OnEditSelectConnected();
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnUpdateEditAddCorner(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditPaste(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSelectedUnlocked(CCmdUI* pCmdUI);
	afx_msg void OnUpdateUnlocked(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewGrid(CCmdUI* pCmdUI);
	afx_msg void OnViewGrid();
	afx_msg void OnWindowRefresh();
	//}}AFX_MSG
  afx_msg void OnEditProperties(UINT cmd);
  afx_msg int  OnMouseActivate(CWnd*, UINT, UINT);
  afx_msg void OnNavGo();
  afx_msg void OnNavGoDir(UINT cmd);
  afx_msg void OnUpdateViewZoom(CCmdUI* pCmdUI);
  afx_msg void OnViewZoom(UINT cmd);
  DECLARE_MESSAGE_MAP();
}; // end CMapView

#ifndef _DEBUG  // debug versions in MapView.cpp
inline const CMapDoc* CMapView::GetDocument() const
   { return (CMapDoc*)m_pDocument; }

inline CMapDoc* CMapView::GetDocument()
   { return (CMapDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////
// Local Variables:
// mode: c++
// compile-command: "nmake -f GUEmap.mak"
// End:
