//--------------------------------------------------------------------
// $Id: Properties.cpp 4718 2007-09-15 02:34:13Z cjm $
//--------------------------------------------------------------------
//
//   GUEmap
//   Copyright 1996-2007 by Christopher J. Madsen
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License along
//   with this program; if not, write to the Free Software Foundation, Inc.,
//   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//   Implementation of the properties dialogs
//
//--------------------------------------------------------------------

#include "StdAfx.hpp"
#include "GUEmap.hpp"
#include "Properties.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// CMapPropPage property page

CMapPropPage::CMapPropPage() : CPropertyPage(CMapPropPage::IDD)
{
	//{{AFX_DATA_INIT(CMapPropPage)
	corners = FALSE;
	grid = FALSE;
	locked = FALSE;
	pages = FALSE;
	name = _T("");
	note = _T("");
	//}}AFX_DATA_INIT
}

void CMapPropPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMapPropPage)
	DDX_Check(pDX, IDC_MP_CORNERS, corners);
	DDX_Check(pDX, IDC_MP_GRID, grid);
	DDX_Check(pDX, IDC_MP_LOCKED, locked);
	DDX_Check(pDX, IDC_MP_PAGES, pages);
	DDX_Text(pDX, IDC_MP_COMMENTS, note);
	DDX_Text(pDX, IDC_MP_TITLE, name);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMapPropPage, CPropertyPage)
	//{{AFX_MSG_MAP(CMapPropPage)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP();

/////////////////////////////////////////////////////////////////////////////
// CMapPropPage message handlers



/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// CRoomPropPage property page

CRoomPropPage::CRoomPropPage() : CPropertyPage(CRoomPropPage::IDD)
{
	//{{AFX_DATA_INIT(CRoomPropPage)
	name = _T("");
	note = _T("");
	border = FALSE;
	//}}AFX_DATA_INIT
}

void CRoomPropPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRoomPropPage)
	DDX_Text(pDX, IDC_ROOM_NAME, name);
	DDX_Text(pDX, IDC_ROOM_NOTES, note);
	DDX_Check(pDX, IDC_ROOM_BORDER, border);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CRoomPropPage, CPropertyPage)
	//{{AFX_MSG_MAP(CRoomPropPage)
	ON_BN_CLICKED(IDC_FILL, OnFillComments)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP();

/////////////////////////////////////////////////////////////////////////////
// CRoomPropPage message handlers
//---------------------------------------------------------------------------
// Gross hack to make Enter & Escape work like they should:

BOOL CRoomPropPage::PreTranslateMessage(MSG* pMsg)
{
  if (CPropertyPage::PreTranslateMessage(pMsg))
    return TRUE;

  if (pMsg->message == WM_KEYDOWN &&
      GetAsyncKeyState(VK_CONTROL) >= 0) { // Control is not pressed
    const int  id = ::GetDlgCtrlID(::GetFocus());
    UINT cmd = 0;

    if (pMsg->wParam == VK_RETURN && id == IDC_ROOM_NAME)
      cmd = IDOK;               // Enter pressed in Name field
    else if (pMsg->wParam == VK_ESCAPE &&
             (id == IDC_ROOM_NAME || id == IDC_ROOM_NOTES) &&
             okToCancel())
      cmd = IDCANCEL;           // Escape pressed in edit field

    if (cmd) {
      CWnd *const  pWnd = GetParent();
      ASSERT(pWnd);
      pWnd->SendMessage(WM_COMMAND, cmd,
                        reinterpret_cast<long>(::GetDlgItem(pWnd->m_hWnd,cmd)));
      return TRUE;
    } // end if sending command message
  } // end if key pressed without Ctrl

  return FALSE;
} // end CRoomPropPage::PreTranslateMessage

//--------------------------------------------------------------------
// See if it's okay to cancel the dialog:
//
// If the name or note has changed, prompt the user to confirm.
//
// Returns:
//   true:   You can cancel the dialog
//   false:  The dialog should not be cancelled

bool CRoomPropPage::okToCancel()
{
  CString  oldName(name);
  CString  oldNote(note);

  UpdateData(TRUE);
  if ((oldName != name || oldNote != note) &&
      AfxMessageBox(_T("Cancel changes?"),
                    MB_ICONEXCLAMATION|MB_YESNO|MB_DEFBUTTON2) != IDYES) {
    name = oldName;
    note = oldNote;
    return false;
  }

  return true;
} // end CRoomPropPage::okToCancel

//--------------------------------------------------------------------
void CRoomPropPage::OnFillComments()
{
  CEdit*  edit = static_cast<CEdit*>(GetDlgItem(IDC_ROOM_NOTES));
  GotoDlgCtrl(edit);
  CString  text;
  edit->GetWindowText(text);
  String s;
  Strcpy(s, text);
  fillParagraphs(s);
  if (s != LPCTSTR(text)) {
    edit->SetSel(0, -1, TRUE);
    edit->ReplaceSel(s.c_str(), TRUE); // Can undo
  }
} // end CRoomPropPage::OnFillComments

/////////////////////////////////////////////////////////////////////////////
// Local Variables:
// compile-command: "nmake -f GUEmap.mak"
// End:
