//--------------------------------------------------------------------
// $Id: ScrollZoom.hpp 4718 2007-09-15 02:34:13Z cjm $
//--------------------------------------------------------------------
//
//   GUEmap
//   Copyright 1999-2007 by Christopher J. Madsen
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License along
//   with this program; if not, write to the Free Software Foundation, Inc.,
//   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//   Interface of the CScrollZoomView class
//
//--------------------------------------------------------------------

#ifndef __SCROLLZOOM_HPP
#define __SCROLLZOOM_HPP

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

/////////////////////////////////////////////////////////////////////////////
// CScrollZoomView view

class CScrollZoomView : public CView
{
  DECLARE_DYNAMIC(CScrollZoomView);

 protected:
  CScrollZoomView();
  void  init(int logPerInch, const SIZE& theDocSize, const SIZE& lineStep);

  // Attributes
 protected:
  int     displayPerInch;
  int     realPerInch;
  CSize   docSize;
  CPoint  offset;
  CSize   scrollLine;
  CSize   scrollPage;
  CSize   scrollStep;

 public:
  const CPoint&  getScrollPosition() const { return offset; }; // upper corner
  bool           initialized()       const { return realPerInch != 0; };

  // Operations
 public:
  bool  calcScrollPosition(POINT& pt) const;
  void  scrollToPosition(const POINT& pt);    // set upper left position
  void  setDisplayZoom(int percent);
  void  setPageStep(const SIZE& step) { scrollPage = step; };

  // Overrides
  // ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CScrollZoomView)
	public:
	virtual void OnPrepareDC(CDC* pDC, CPrintInfo* pInfo = NULL);
	virtual BOOL OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll = TRUE);
	protected:
	virtual BOOL OnScrollBy(CSize sizeScroll, BOOL bDoScroll = TRUE);
	//}}AFX_VIRTUAL

  // Implementation
 private:
  void  setDisplayScale(int logPerInch);
  void  updateScrollBars();

 protected:
#ifdef _DEBUG
  virtual void AssertValid() const;
  virtual void Dump(CDumpContext& dc) const;
#endif

  // Generated message map functions
 protected:
	//{{AFX_MSG(CScrollZoomView)
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	//}}AFX_MSG
  DECLARE_MESSAGE_MAP();
}; // end CScrollZoomView

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}

#endif // not __SCROLLZOOM_HPP
