//--------------------------------------------------------------------
// $Id: Undo.hpp 4718 2007-09-15 02:34:13Z cjm $
//--------------------------------------------------------------------
//
//   GUEmap
//   Copyright 1997-2007 by Christopher J. Madsen
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License along
//   with this program; if not, write to the Free Software Foundation, Inc.,
//   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//   Undo.h : interface of UndoRec and its derived classes
//
//--------------------------------------------------------------------

#ifndef __UNDO_H

#define __UNDO_H

// Requires "MapDoc.h"

class UndoRec
{
 public:
  bool  oldModified;
 public:
  UndoRec(bool modified) : oldModified(modified) {};
  virtual ~UndoRec() {};
  // Functions unique to UndoRec:
  UndoRec*  undo(CMapDoc& doc);
  virtual LPCTSTR  getName() const = 0;
  virtual void  undoChange(CMapDoc& doc, UndoRec*& redo) = 0;
}; // end UndoRec

class UndoAdd : public UndoRec
{
 protected:
  int         edges;
  PageNumVec  pageNums;
  RoomNumVec  roomNums;
 public:
  UndoAdd(bool modified, int edgeCount, RoomNumVec& theRoomNums,
          PageNumVec& thePageNums);
  UndoAdd(bool modified, RoomNum roomNum, int edgeCount);
  UndoAdd(bool modified, short pageNum);
  UndoAdd(bool modified);
  // Overloaded functions:
  LPCTSTR  getName() const;
  void  undoChange(CMapDoc& doc, UndoRec*& redo);
}; // end UndoAdd

class UndoChangeEdge : public UndoRec
{
 protected:
  MapEdge  edge;
 public:
  UndoChangeEdge(bool modified, const MapEdge& theEdge);
  // Overloaded functions:
  LPCTSTR  getName() const;
  void  undoChange(CMapDoc& doc, UndoRec*& redo);
}; // end UndoChangeEdge

class UndoDelete : public UndoRec
{
 protected:
  EdgeVec     edges;
  PageVec     pages;
  PageNumVec  pageNums;
  RoomArray   rooms;
  RoomNumVec  roomNums;
 public:
  UndoDelete(const CMapDoc& doc, int numSelected, const ByteVec& selectedRooms,
             int numSelectedPages, const ByteVec& selectedPages);
  UndoDelete(const CMapDoc& doc, int numEdges, const RoomNumVec& theRoomNums,
             const PageNumVec& thePageNums);
  UndoDelete(bool modified, const MapEdge& edge);
  UndoDelete(bool modified, EdgeVec& theEdges);
  void addRoom(VecSize pos, RoomNum n, MapRoom* r)
    { roomNums[pos] = n; rooms.getVector()[pos] = r; };
  void addRooms(VecSize n) { rooms.resize(n); roomNums.resize(n); };
  // Overloaded functions:
  LPCTSTR  getName() const;
  void  undoChange(CMapDoc& doc, UndoRec*& redo);
}; // end UndoDelete

class UndoChanges : public UndoRec
{
 protected:
  UndoRec*  add;
  UndoRec*  del;
 public:
  UndoChanges(bool modified, RoomNum roomNum, int edgeCount,
              const MapEdge& edge);
  UndoChanges(bool modified, int roomNum, int edgeCount, EdgeVec& edges);
  UndoChanges(bool modified, RoomNum roomNum, MapRoom* room, EdgeVec& edges);
  // Overloaded functions:
  LPCTSTR  getName() const;
  void  undoChange(CMapDoc& doc, UndoRec*& redo);
}; // end UndoChanges

class UndoMove : public UndoRec
{
 protected:
  CSize       offset;
  PageNumVec  pageNums;
  RoomNumVec  roomNums;
 public:
  UndoMove(bool modified, const CSize& moved, int numSelected,
           const ByteVec& selectedRooms, int numSelectedPages,
           const ByteVec& selectedPages);
  UndoMove(bool modified, const CSize& moved, RoomNumVec& rooms,
           PageNumVec& pages);
  // Overloaded functions:
  LPCTSTR  getName() const;
  void  undoChange(CMapDoc& doc, UndoRec*& redo);
}; // end UndoMove

class UndoPaginate : public UndoRec
{
 protected:
  PageVec  pages;
 public:
  UndoPaginate(CMapDoc& doc);
  const PageVec&  getPages() const { return pages; };
  // Overloaded functions:
  LPCTSTR  getName() const;
  void  undoChange(CMapDoc& doc, UndoRec*& redo);
}; // end UndoPaginate

class UndoRoomInfo : public UndoRec
{
 protected:
  RoomNum  num;
  MapRoom  room;
 public:
  UndoRoomInfo(const CMapDoc& doc, RoomNum roomNum);
  // Functions unique to UndoRoomInfo:
  const MapRoom&  getRoom() const { return room; };
  // Overloaded functions:
  LPCTSTR  getName() const;
  void  undoChange(CMapDoc& doc, UndoRec*& redo);
}; // end UndoRoomInfo

#endif // __UNDO_H

/////////////////////////////////////////////////////////////////////////////
// Local Variables:
// mode: c++
// compile-command: "nmake -f GUEmap.mak"
// End:
